#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>

/*
 * Questo programma fa uso, per calcolare mese e giorno correnti, della
 * struct tm, definita in time.h e contenente i seguenti campi.
 *
 * struct tm
 * {
 * 	int tm_sec; //   seconds after the minute — [0, 60] 
 * 	int tm_min;    minutes after the hour — [0, 59] 
 * 	int tm_hour;   hours since midnight — [0, 23] 
 * 	int tm_mday;   day of the month — [1, 31] 
 * 	int tm_mon;    months since January — [0, 11] 
 * 	int tm_year;   years since 1900 
 * 	int tm_wday;   days since Sunday — [0, 6] 
 * 	int tm_yday;   days since January 1 — [0, 365]
 * 	int tm_isdst;  Daylight Saving Time flag
 * };
 *
 */

#define MAXLEN 1024
#define MAXLINELEN 2 * MAXLEN /* lunghezza massima di una riga nel file dei dati */
#define MAXNOTE 2048

struct nota
{
	int giorno, mese;
	char nota[MAXLEN];
};

struct nota leggi_nota();

int aggiungi_nota(struct nota n);

int carica_note(struct nota note[]);

int main()
{
	/* inizializzo solo la prima nota del vettore, tutte le altre 
	 * automaticamente inizializzate con tutti i campi a 0 (vedi
	 * inizializzazione di vettori. Le due parentesi piu` interne
	 * delimitano la prima struttura.
	 */
	struct nota note[MAXNOTE] = { { 0, 0, "" } };
	struct nota nuova_nota;
	time_t now;
	struct tm *tm_now;
	int i;
	char c;

	time(&now);
	tm_now = localtime ( &now );
	
	if(carica_note(note) < 0)
	{
		printf("Errore in carica_note()\n");
		return EXIT_FAILURE;
	}

	/* 1. vediamo se ci sono note per oggi.
	 *    Ci fermiamo quando il contatore raggiunge MAXNOTE oppure quando troviamo
	 *    la nota 'delimitatore', che ha tutti i campi a zero, tra cui il giorno.
	 */
	for(i = 0; i < MAXNOTE && note[i].giorno != 0; i++)
	{
		if(note[i].giorno == (*tm_now).tm_mday	&& note[i].mese == (*tm_now).tm_mon + 1) /* tm_mon va da 0 a 11 ... */
			printf("* nota per oggi: %s\n", note[i].nota);
	}

	/* 2. chiediamo all'utente se vuole aggiungere una nota */
	do
	{
		printf("Vuoi aggiungere una nota [s|n]? ");
		c = getchar();
		if(c == 's')
		{
			nuova_nota = leggi_nota();
			if(aggiungi_nota(nuova_nota) < 0)
			{
				printf("Errore in aggiungi_nota()\n");
				return EXIT_FAILURE;
			}
			printf("* nota aggiunta per il giorno %d/%d\n", nuova_nota.giorno, nuova_nota.mese);
		}
	}while(c == 's');

	return EXIT_SUCCESS;
	
}

struct nota leggi_nota()
{
	struct nota n;
	printf("Inserisci giorno: ");
	scanf("%d", &n.giorno);
	printf("Inserisci mese: ");
	scanf("%d", &n.mese);
	while(getchar() != '\n')
		;
	printf("Inserisci il testo della nota:\n");
	fgets(n.nota, MAXLEN, stdin);
	n.nota[strlen(n.nota) - 1] = '\0'; /* rimuovo newline */

	return n;
}

int aggiungi_nota(struct nota n)
{
	FILE *fp;
	fp = fopen("calendario.txt", "a");
	if(!fp)
	{
		printf("Errore di apertura del file \"calendario.txt\" in modalita` append\n");
		return -1; /* la funzione esce qui restituendo un codice di errore */
	}
	fprintf(fp, "%d %d:%s\n", n.giorno, n.mese, n.nota);
	fclose(fp);
	return 0; /* ok */
}

/* legge le note dal file calendario.txt.
 * si suppone che il formato delle note sia il seguente:
 * giorno mese:Testo della nota
 * ad esempio
 * 1 12:Ultima lezione di Esercitazioni di Fondamenti di Informatica.
 */
int carica_note(struct nota note[])
{
	char line[MAXLINELEN];
	int i = 0, stridx;
	FILE *fp = fopen("calendario.txt", "r");
	if(fp == NULL)
	{
		printf("Errore di apertura del file \"calendario.txt\" in sola lettura!\n");
		return -1;
	}

	while(fgets(line, MAXLINELEN, fp) != NULL) /* fgets ritorna NULL quando incontra EOF */
	{
		stridx = 0;
		line[strlen(line) - 1] = '\0'; /* rimuovo il newline */
		sscanf(line, "%d %d", &note[i].giorno, &note[i].mese);
		while(line[stridx] != ':' && stridx < strlen(line)) /* trovo i ':' che separano giorno e mese da nota */
			stridx++;
		/* copio il contenuto della linea a partire dal carattere che segue i ':'
		 * line + stridx + 1 e` il puntatore al carattere successivo ai ':' trovato nel
		 * ciclo precedente.
		 * Ricordo che, come discusso piu` volte a lezione:
		 * - line e` un puntatore a carattere e contiene l'indirizzo del primo carattere contenuto in line;
		 * - l'operatore + e` applicato a puntatori quindi e` dimensionato al size di un char
		 * - ad es, line + 3 puntera` al quarto carattere
		 * - se la nostra line e` siffatta:
		 *   "25 12:Buon Natale\n"
		 *   - stridx vale 5 (punta alla locazione che contiene ':' (sesto carattere);
		 *   - stridx + 1 punta al settimo carattere, 'B'
		 *   - strcpy(note[i].nota, line + stridx + 1) copia dentro la stringa dell'i-ma nota
		 *     la stringa contenuta il line a partire al settimo carattere, la 'B' di "Buon Natale\n"
		 */
		strcpy(note[i].nota, line + stridx + 1);

		/* NOTA: la riga sopra equivale a questa sotto */
//		strcpy(note[i].nota, &line[stridx + 1]);
		i++;
	}
	
	fclose(fp);
	return 0;
}


