/*
 * FUNZIONI USATE sulle stringhe: 
 *
 * 1. strcat
 
NAME
       strcat - concatenate two strings

SYNOPSIS
       #include <string.h>

       char *strcat(char *restrict s1, const char *restrict s2);


DESCRIPTION
       The strcat() function shall append a copy of the string pointed to by s2 (including the terminating null byte) to the end of the string  pointed  to
       by  s1. The initial byte of s2 overwrites the null byte at the end of s1. If copying takes place between objects that overlap, the behavior is unde‐
       fined.

RETURN VALUE
       The strcat() function shall return s1; no return value is reserved to indicate an error.
 *
 *
 * 2. strstr

NAME
       strstr - find a substring

SYNOPSIS
       #include <string.h>

       char *strstr(const char *s1, const char *s2);


DESCRIPTION
       The strstr() function shall locate the first occurrence in the string pointed to by s1 of the sequence of  bytes  (excluding  the  terminating  null
       byte) in the string pointed to by s2.

RETURN VALUE
       Upon successful completion, strstr() shall return a pointer to the located string or a null pointer if the string is not found.

       If s2 points to a string with zero length, the function shall return s1.


*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define MAXLINELEN 512
#define MAX_LINEE_PER_CAP 25

int estrai(char *filename, char* keyword);

int contiene_parola_chiave(char *filename, char* keyword);

int scrivi_capoverso(char *capoverso, char *keyword, int id);

int main()
{
	int count;
	char filename[256];
	char keyword[256];
	printf("Che file vuoi esaminare? ");
	scanf("%s", filename);
	printf("Inserisci la parola chiave che vuoi cercare: " );
	scanf("%s", keyword);
	
	count = estrai(filename, keyword);

	printf("Estratti %d capoversi contenenti la parola chiave \"%s\" da \"%s\"\n", count, keyword, filename);
	
	if(count >= 0)
		return EXIT_SUCCESS;
	return EXIT_FAILURE;
}

int scrivi_capoverso(char *capoverso, char *keyword, int id)
{
	FILE *fout;
	char nomefile[32];
	/* sprintf: lo stesso che printf solo che sulla stringa passata come primo parametro.
	 * Voglio preporre un id al nome del file per poterli distinguere tra di loro.
	 * L'id viene passato come parametro di ingresso della funzione e contiene il numero
	 * d'ordine del capoverso estratto.
	 */
	 
	sprintf(nomefile, "%d_%s.txt", id, keyword);

	printf("salvando \"%s\"...\n", nomefile);
	fout = fopen(nomefile, "w");
	if(fout != NULL)
	{
		fputs(capoverso, fout);
		fclose(fout);
		return 1;
	}
	else
	{
		printf("Errore di apertura del file \"%s\" in scrittura!\n", nomefile);
		return -1;
	}
}

/* questa funzione apre il file "filename" in sola lettura. Tramite la fgets() legge le linee
 * contenute nel file. Ogni linea e` salvata in una stringa chiamata capoverso abbastanza lunga
 * da contenere la lunghezza di una linea (MAXLINELEN) moltiplicato il numero di linee massimo
 * che possiamo aspettarci per un capoverso (MAX_LINEE_PER_CAP).
 * Quando incontro i caratteri "punto e a capo" (".\n"), considero il capoverso terminato.
 * Invoco quindi la scrivi_capoverso, che scrive su file il capoverso estratto e poi
 * cancello il contenuto di capoverso ponendo uguale a '\0' il suo primo carattere.
 *
 * Nella fase di individuazione del punto e a capo sorge un problema (quello riscontrato in
 * classe al momento dell'esecuzione del programma):
 * - il carattere di fine linea UNIX/Linux e` il "feed" o "newline", quello che noi abbiamo sempre
 *   usato ('\n');
 * - DOS/Windows usa invece il carriage return ('\r') seguito da newline ('\n'). Quindi la fine 
 *   linea di un file DOS/Windows e` marcata da "\r\n".
 *
 * La prima versione del programma, alla riga
 * if(strlen(capoverso) > 2 && capoverso[strlen(capoverso) - 2] == '.')
 * non trovava mai il punto quando il file era di tipo dos/windows, perche` alla locazione 
 * [strlen(capoverso) - 2] incontrava il '\r'.
 * Il punto trovavasi alla locazione [strlen(capoverso) - 3].
 *
 * Questa versione di estrai() controlla quindi la natura della fine linea per funzionare in
 * entrambi i casi. La variabile `posizione_punto' indica la posizione a partire dalla fine
 * in cui cercare il punto. Varra` quindi 2 oppure 3 a seconda di come la linea viene terminata
 * ('\n') nel caso a noi noto, oppure "\r\n" nel caso DOS/Windows.
 */
int estrai(char *filename, char* keyword)
{
	int estratti = 0;	
	FILE *fp = fopen(filename, "r");
	char line[MAXLINELEN];
	char capoverso[MAXLINELEN * MAX_LINEE_PER_CAP];
	char posizione_punto;

	if(fp != NULL)
	{
		while(fgets(line, MAXLINELEN, fp) != NULL)
		{		
			/* la fgets() salva anche il '\n' */
			/* rendiamo piu` robusto il programma se controlliamo che 
			 * strlen(capoverso) + strlen(line) < MAXLINELEN * MAX_LINEE_PER_CAP -1
			 * il -1 perche` poi serve lo spazio per il terminatore di fine stringa ('\0').
			 * 
			 */
			strcat(capoverso, line); /* si veda commento al principio del file */
			/* adesso devo cercare il '.' prima del '\n' */
			if(capoverso[strlen(capoverso) - 2] == '\r')
				posizione_punto = 3;
			else
				posizione_punto = 2;
			if(strlen(capoverso) > posizione_punto && capoverso[strlen(capoverso) - posizione_punto] == '.')
			{
				if(strstr(capoverso, keyword)) /* strstr spiegata nel commento all'inizio del file */
					estratti += scrivi_capoverso(capoverso, keyword, estratti);
				/* cancella capoverso */
				capoverso[0] = '\0';
			}
		}
		/* chiudo il file */
		fclose(fp);
		return estratti;
	}
	else
	{
		printf("Errore di apertura del file \"%s\" in sola lettura.\n", filename);
		return -1;
	}
}

