
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define SMSCOMPRDATA "SMScompress.dat"
#define SMSLEN 512
#define MAXSTRLEN     32
#define MAX_SMS_SUBS  64


struct sms_subst
{
   char original[MAXSTRLEN];
   char compressed[MAXSTRLEN];
};

/* legge dal file "filename" le sostituzioni e le salva nel vettore di strutture
 * passato come parametro in ingresso.
 */
int load_sms_subst (char filename[], struct sms_subst v[]);

/* comprime l'SMS letto dal file "filename" e lo salva nel vettore di caratteri
 * smsout.
 */
int SMScompress (char *filename, char *smsin, char *smsout);


int main ()
{
   char smscompresso[SMSLEN];
   char smsin[SMSLEN];
   int success160;
   
   printf("Inserisci il testo dell'SMS da comprimere: \n");
   fgets(smsin, SMSLEN, stdin);

   success160 = SMScompress (SMSCOMPRDATA, smsin, smscompresso);
   
   if(success160 < 0)
   	printf("Errore in SMScompress\n");
   else if(success160 <= 160)
   	printf("\nComplimenti! Sei riuscito a comprimere l'SMS da %d a %d caratteri  :D \n", strlen(smsin), success160);
   else
   	printf("\n:-( nonostante la compressione da %d a %d caratteri, l'SMS e' troppo lungo: dovrai spedirne %d\n", strlen(smsin), success160, success160 / 160 + 1);
   
   printf ("\nSMS compresso: %s\n\n", smscompresso);

   return EXIT_SUCCESS;
}

int SMScompress (char *filename, char *smsin, char *smsout) 
{ 
   /* vettore di strutture sms_subst, cioe' vettore di coppie parola/sostituzione */
   struct sms_subst sms_subst_vector[MAX_SMS_SUBS];
   int n_subst;
   int i;
   int inidx = 0, outidx = 0;
   unsigned char found;

   /* carica le coppie delle sostituzioni dal file filename e le salva nel vettore sms_subst_vector */
   n_subst = load_sms_subst (filename, sms_subst_vector);
   
   if(n_subst < 0)
   	return -1; /* la funzione termina qui restituendo un codice di errore */

   /* smsout inizialmente e' la stringa nulla */
   smsout[outidx] = '\0';

   while (smsin[inidx] != '\0')
   {
      i = 0;
      found = 0;
      while (!found && i < n_subst)
      {
      	 /* smsin e' il puntatore al primo carattere della stringa.
      	  * smsin + inidx punta all'inidx-esimo carattere. E' una somma di puntatori
      	  * dove l'operatore binario + e' dimensionato al size di un char (rivedi lezione
      	  * sui puntatori).
      	  * Quindi partendo dalla posizione inidx della stringa, cerco un'occorrenza di una parola
      	  * tra quelle da sostituire, finche' non ne trovo una (found = 1). Il contatore i avanza
      	  * all'interno dell'elenco delle parole da sostituire puntando alla parola successiva.
      	  */
         if (strncmp (smsin + inidx, sms_subst_vector[i].original,
                       strlen (sms_subst_vector[i].original)) == 0)
            found = 1;
         else
            i++;
      }

      if (found) 
      {
      	 /* concateno a smsout la parola compressa al posto della originale trovata in smsin da strncmp sopra */
         strcat (smsout, sms_subst_vector[i].compressed);
         inidx += strlen (sms_subst_vector[i].original); /* muovo la posizione dell'indice di sms in */
         outidx += strlen (sms_subst_vector[i].compressed); /* muovo la posizione dell'indice di sms out */
      }
      else /* semplicemente copio un carattere da smsin a smsout */
      {
         smsout[outidx] = smsin[inidx];
         outidx++;
         inidx++;
         smsout[outidx] = '\0'; /* sposto in avanti di uno il terminatore di smsout */
      }
   }

   smsout[outidx] = '\0';

   return strlen(smsout);
}


int load_sms_subst (char *filename, struct sms_subst v[])
{
   FILE *fin;
   int i;
   
   fin = fopen(filename, "r");

   if (fin == NULL)
   {
      printf ("errore di apertura del file di definizione "
              "delle sostituzioni\n");
      return -1;
   }

   i = 0;

   while (i < MAX_SMS_SUBS && 
          fscanf (fin, "%s %s", v[i].original, v[i].compressed) != EOF)
      i++;
   /* chiudo il file */   
   fclose(fin);
   return i;
}
